# -*- coding: utf-8 -*-

# Copyright 2015-2018 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extract images from https://www.hentai-foundry.com/"""

from .common import Extractor, Message
from .. import text, util, exception


class HentaifoundryExtractor(Extractor):
    category = "hentaifoundry"
    directory_fmt = ["{category}", "{artist}"]
    filename_fmt = "{category}_{index}_{title}.{extension}"
    archive_fmt = "{index}"
    root = "https://www.hentai-foundry.com"
    per_page = 25

    def __init__(self, artist):
        Extractor.__init__(self)
        self.artist = artist
        self.artist_url = "{}/pictures/user/{}".format(self.root, self.artist)
        self.start_post = 0

    def items(self):
        data = self.get_job_metadata()
        yield Message.Version, 1
        yield Message.Directory, data

        for page_url in util.advance(self.get_image_pages(), self.start_post):
            url, image = self.get_image_metadata(page_url)
            image.update(data)
            if not image["extension"]:
                image["extension"] = "jpg"
            yield Message.Url, url, image

    def get_image_pages(self):
        """Yield urls all image pages of one artist"""

    def get_job_metadata(self):
        """Collect metadata for extractor-job"""

    def get_image_metadata(self, url):
        """Collect metadata for an image"""
        page = self.request(text.urljoin(self.root, url)).text
        index = url.rsplit("/", 2)[1]
        title, pos = text.extract(
            page, 'Pictures</a> &raquo; <span>', '<')
        part, pos = text.extract(
            page, '//pictures.hentai-foundry.com', '"', pos)
        data = {"index": text.parse_int(index), "title": text.unescape(title)}
        text.nameext_from_url(part, data)
        return "https://pictures.hentai-foundry.com" + part, data


class HentaifoundryUserExtractor(HentaifoundryExtractor):
    """Extractor for all images of a hentai-foundry-user"""
    subcategory = "user"
    pattern = [r"(?:https?://)?(?:www\.)?hentai-foundry\.com/"
               r"(?:pictures/user/([^/]+)(?:/(?:page/(\d+))?)?$"
               r"|user/([^/]+)/profile)"]
    test = [
        ("https://www.hentai-foundry.com/pictures/user/Tenpura", {
            "url": "ebbc981a85073745e3ca64a0f2ab31fab967fc28",
            "keyword": "f8fecc8aa89978ecf402ec221243978fe791bd54",
        }),
        ("http://www.hentai-foundry.com/user/asdq/profile", {
            "exception": exception.NotFoundError,
        }),
        ("https://www.hentai-foundry.com/pictures/user/Tenpura/page/3", None),
    ]

    def __init__(self, match):
        HentaifoundryExtractor.__init__(self, match.group(1) or match.group(3))
        self.start_page = text.parse_int(match.group(2), 1)
        self._skipped = (self.start_page - 1) * self.per_page

    def skip(self, num):
        pages, posts = divmod(num, self.per_page)
        self.start_page += pages
        self.start_post += posts
        self._skipped += num
        return num

    def get_image_pages(self):
        num = self.start_page

        while True:
            url = "{}/page/{}".format(self.artist_url, num)
            page = self.request(url).text
            yield from text.extract_iter(page, 'thumbTitle"><a href="', '"')

            if 'class="pager"' not in page or 'class="last hidden"' in page:
                return
            num += 1

    def get_job_metadata(self):
        url = self.artist_url + "?enterAgree=1"
        response = self.request(url, expect=(404,))

        if response.status_code == 404:
            raise exception.NotFoundError("user")
        count = text.parse_int(text.extract(
            response.text, 'class="active" >Pictures (', ')')[0])
        if self._skipped >= count:
            raise exception.StopExtraction()

        self.set_filters()
        return {"artist": self.artist, "count": count}

    def set_filters(self):
        """Set site-internal filters to show all images"""
        token = text.extract(
            self.session.cookies["YII_CSRF_TOKEN"], "%22", "%22")[0]
        formdata = {
            "YII_CSRF_TOKEN": token,
            "rating_nudity": 3,
            "rating_violence": 3,
            "rating_profanity": 3,
            "rating_racism": 3,
            "rating_sex": 3,
            "rating_spoilers": 3,
            "rating_yaoi": 1,
            "rating_yuri": 1,
            "rating_teen": 1,
            "rating_guro": 1,
            "rating_furry": 1,
            "rating_beast": 1,
            "rating_male": 1,
            "rating_female": 1,
            "rating_futa": 1,
            "rating_other": 1,
            "rating_scat": 1,
            "rating_incest": 1,
            "rating_rape": 1,
            "filter_media": "A",
            "filter_order": "date_new",
            "filter_type": 0,
        }
        self.request("https://www.hentai-foundry.com/site/filters",
                     method="post", data=formdata)


class HentaifoundryImageExtractor(HentaifoundryExtractor):
    """Extractor for a single image from hentaifoundry.com"""
    subcategory = "image"
    pattern = [(r"(?:https?://)?(?:www\.|pictures\.)?hentai-foundry\.com/"
                r"(?:pictures/user/([^/]+)/(\d+)"
                r"|[^/]/([^/]+)/(\d+))")]
    test = [
        (("http://www.hentai-foundry.com"
          "/pictures/user/Tenpura/407501/shimakaze"), {
            "url": "fbf2fd74906738094e2575d2728e8dc3de18a8a3",
            "keyword": "2956321893e9187edde4aeac6bed889449692e6a",
            "content": "91bf01497c39254b6dfb234a18e8f01629c77fd1",
        }),
        ("http://www.hentai-foundry.com/pictures/user/Tenpura/340853/", {
            "exception": exception.HttpError,
        }),
    ]

    def __init__(self, match):
        HentaifoundryExtractor.__init__(self, match.group(1) or match.group(3))
        self.index = match.group(2) or match.group(4)

    def get_image_pages(self):
        return ("{}/{}?enterAgree=1".format(self.artist_url, self.index),)

    def get_job_metadata(self):
        return {"artist": self.artist, "index": text.parse_int(self.index)}



# -*- coding: utf-8 -*-

# Copyright 2018 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extract manga-chapters and entire manga from https://komikcast.com/"""

from .common import ChapterExtractor, MangaExtractor
from .. import text, cloudflare
import re


class KomikcastBase():
    """Base class for komikcast extractors"""
    category = "komikcast"
    scheme = "https"
    root = "https://komikcast.com"

    request = cloudflare.request_func

    @staticmethod
    def parse_chapter_string(chapter_string, data=None):
        """Parse 'chapter_string' value and add its info to 'data'"""
        if not data:
            data = {}

        match = re.match(
            r"(?:(.*) Chapter )?0*(\d+)([^ ]*)(?: (?:- )?(.+))?",
            text.unescape(chapter_string),
        )
        manga, chapter, data["chapter_minor"], title = match.groups()

        if manga:
            data["manga"] = manga.partition(" Chapter ")[0]
        if title and title.lower() != "bahasa indonesia":
            data["title"] = title.strip()
        else:
            data["title"] = ""
        data["chapter"] = text.parse_int(chapter)
        data["lang"] = "id"
        data["language"] = "Indonesian"

        return data


class KomikcastChapterExtractor(KomikcastBase, ChapterExtractor):
    """Extractor for manga-chapters from komikcast.com"""
    pattern = [r"(?:https?://)?(?:www\.)?komikcast\.com(/chapter/[^/?&#]+/)"]
    test = [
        (("https://komikcast.com/chapter/"
          "apotheosis-chapter-02-2-bahasa-indonesia/"), {
            "url": "978d3c053d34a77f6ea6e60cbba3deda1e369be8",
            "keyword": "9964a7ce7c8a518aebdccdea0e05858439c7ad92",
        }),
        (("https://komikcast.com/chapter/"
          "tonari-no-kashiwagi-san-chapter-18b/"), {
            "url": "db5594b025f9d81e4987da538b8599b8dee8851b",
            "keyword": "94bb85aec6654ab5af0c10419ca388fcd9c73b47",
        }),
    ]

    def __init__(self, match):
        ChapterExtractor.__init__(self, self.root + match.group(1))

    def get_metadata(self, page):
        info = text.extract(page, '<b>', "</b>")[0]
        return self.parse_chapter_string(info)

    @staticmethod
    def get_images(page):
        readerarea = text.extract(
            page, '<div id="readerarea">', '<div class="navig">')[0]
        return [
            (text.unescape(url), {
                "width": text.parse_int(width),
                "height": text.parse_int(height),
            })
            for url, width, height in re.findall(
                r"<img[^>]*? src=[\"']([^\"']+)[\"']"
                r"(?:\s+data-original-width=[\"']([^\"']+)[\"'])?"
                r"(?:\s+data-original-height=[\"']([^\"']+)[\"'])?",
                readerarea
            )
            if "/Banner-" not in url
        ]


class KomikcastMangaExtractor(KomikcastBase, MangaExtractor):
    """Extractor for manga from komikcast.com"""
    pattern = [r"(?:https?://)?(?:www\.)?(komikcast\.com"
               r"/(?:komik/)?[^/?&#]+/?)$"]
    test = [
        ("https://komikcast.com/komik/090-eko-to-issho/", {
            "url": "dc798d107697d1f2309b14ca24ca9dba30c6600f",
            "keyword": "3db7e23e3c108031608fbbeb9334badecd967f95",
        }),
        ("https://komikcast.com/tonari-no-kashiwagi-san/", None),
    ]

    def chapters(self, page):
        results = []
        data = self.get_metadata(page)

        for item in text.extract_iter(
                page, '<span class="leftoff"><a href="', '</a>'):
            url, _, chapter_string = item.rpartition('">Chapter ')
            self.parse_chapter_string(chapter_string, data)
            results.append((url, data.copy()))
        return results

    @staticmethod
    def get_metadata(page):
        """Return a dict with general metadata"""
        manga , pos = text.extract(page, "<title>", "</title>")
        author, pos = text.extract(page, "<th>Author</th><td>", "</td>", pos)
        genres, pos = text.extract(page, "<th>Genres </th><td>", "</td>", pos)
        mtype , pos = text.extract(page, "<th>Type </th><td>", "</td>", pos)

        return {
            "manga": text.unescape(manga.rpartition(" - ")[0]),
            "author": text.unescape(author),
            "genres": text.remove_html(genres).replace(" , ", ", "),
            "type": text.remove_html(mtype),
        }



# -*- coding: utf-8 -*-

# Copyright 2017 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extractors for https://reader.kireicake.com/"""

from . import foolslide


class KireicakeChapterExtractor(foolslide.FoolslideChapterExtractor):
    """Extractor for manga-chapters from reader.kireicake.com"""
    category = "kireicake"
    pattern = foolslide.chapter_pattern(r"reader\.kireicake\.com")
    test = [("https://reader.kireicake.com/read/wonderland/en/1/1/", {
        "url": "b2d36bc0bc67e4c461c3a4d6444a2fd339f5d07e",
        "keyword": "47e0cf69f95ab3b820bda05014aec38d3b824018",
    })]


class KireicakeMangaExtractor(foolslide.FoolslideMangaExtractor):
    """Extractor for manga from reader.kireicake.com"""
    category = "kireicake"
    pattern = foolslide.manga_pattern(r"reader\.kireicake\.com")
    test = [("https://reader.kireicake.com/series/wonderland/", {
        "url": "d067b649af1cc88fa8c8b698fde04a10909fd169",
        "keyword": "99caa336a9d48e27e3b8e56a0a1e6faf9fc13a51",
    })]
